package com.ruoyi.benyun.service;


import com.ruoyi.benyun.entity.bo.*;
import com.ruoyi.benyun.entity.vo.BrandEvolveVo;
import com.ruoyi.benyun.entity.vo.BrandOpcloseVo;
import com.ruoyi.benyun.entity.vo.ListQueryBody;
import com.github.pagehelper.PageInfo;

import java.time.LocalDateTime;
import java.util.List;

public interface BrandService {
    List<BrandSearch> searchByLikeName(String text);
    BrandInfo searchByBrandId(String brandId, String userId);
    BrandProperty searchPropertyByBrandId(String brandId);
    List<BrandSimilar> searchSimilarByBrandId(String brandId);
    List<BrandStatus> searchStatusByBrandId(String brandId);
    BrandDistribution searchDistributionByBrandId(String brandId);
    List<BrandProvinceBo> searchProvinceByBrandId(String brandId);
    BrandDisCloud searchDisCloudByBrandId(String brandId);
    PageInfo<List<BrandSearch>> searchListByMulti(ListQueryBody body, String userId);
    List<BrandTotal> searchTotal(Integer pageSize);
//    BStoreData searchAvgScoreByBrandId(String brandId);
//    List<BrandAddrDistribution> searchAddrDistribution(List<String> addrCodes);
    List<BrandGeoLabelBo> searchGeoLabel(String brandId);
    List<BrandOpcloseVo> searchOpenByMulti(List<String> typeCodes, LocalDateTime startTime, LocalDateTime endTime);
    List<BrandOpcloseVo> searchCloseByMulti(List<String> typeCodes, LocalDateTime startTime, LocalDateTime endTime);
    List<BrandEvolveVo> searchEvolveByMulti(List<String> addrCodes,List<String> typeCodes,List<String> brandIds);
}
